@@ -1,5 +1,5 @@
 # orm/descriptor_props.py
-# Copyright (C) 2005-2016 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2017 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -91,7 +91,7 @@ class CompositeProperty(DescriptorProperty):
     """
 
     def __init__(self, class_, *attrs, **kwargs):
-        """Return a composite column-based property for use with a Mapper.
+        r"""Return a composite column-based property for use with a Mapper.
 
         See the mapping documentation section :ref:`mapper_composite` for a
         full usage example.
@@ -574,7 +574,18 @@ class SynonymProperty(DescriptorProperty):
 
     @util.memoized_property
     def _proxied_property(self):
-        return getattr(self.parent.class_, self.name).property
+        attr = getattr(self.parent.class_, self.name)
+        if not hasattr(attr, 'property') or not \
+                isinstance(attr.property, MapperProperty):
+            raise sa_exc.InvalidRequestError(
+                """synonym() attribute "%s.%s" only supports """
+                """ORM mapped attributes, got %r""" % (
+                    self.parent.class_.__name__,
+                    self.name,
+                    attr
+                )
+            )
+        return attr.property
 
     def _comparator_factory(self, mapper):
         prop = self._proxied_property
